% Continuous time finite horizon lqr
% Finite Horizon Optimal Control of 
% Control dX = a X dt  + b  U dt + sigma dBt
% with r(x,u) = - x'*p*x -  u'*q*u;
% v(x,t)  = min_pi E[ int_t^T e^{-(s-t)/tau} r(X_s, U_s)ds  | X_t= x, pi) ]
% where U_s = pi(X_s)
% 
%  tau = is accepted. 
%
%  Can be shown 
%  v(x,t) = x' alpha_t x + beta_t
%  and optimal action
%  u_t = -k_t x_t where k_t = q^{-1} b' alpha_t
%
% Javier R. Movellan
% UCSD, April 2010
%


function [k,alpha, beta]= ctfhlqr(a,b,c,p,q,tau,dt,T)

  s = ceil(T/dt);
  
  gamma = 1/tau;

 
  qinv = pinv(q);
  alpha = cell(s,1);
  beta = cell(s,1);
  k = cell(s,1);
  
  alpha{s} = p;
  k{s} = qinv*b'*alpha{s};
  nu = length(q);
  beta{s} = zeros(nu,nu);
  for t=s:-1: 2
    alphaBar= (alpha{t}+ alpha{t}')/2;
    dalpha = alpha{t}*gamma -p +  alphaBar*b*qinv*b'*alphaBar  - 2*alphaBar*a ;
    dbeta =  beta{t}*gamma -trace(c'*c*alpha{t}) ;
    alpha{t-1} = alpha{t} - dt*dalpha;
    k{t-1} = qinv*b'*alphaBar;
    beta{t-1} = beta{t} - dt*dbeta;
  end

