% Test the continuous time finite horizon linear quadratic regulator
% 
% Javier R. Movellan
% April 2009
%

clf
clear

% a second order system. x1= state; x2 = velocity of state
%
% dx = (a x + bu) dt + c dBt 
%

nx =2; % dimensionality of the state
nu=1; % diensionality of action

a = [ 0 1; 0 0];

b = [0; 1]; % the action is a force. i.e, it has an effect on the
           % derivative of the velocity

c = 0.0*eye(nx);


q = 1; % action cost

p = [ 10 0 ; 0 0 ];


%J(x) = \int_0^T e^{-t/tau) (x_t'p x_t + u_t'q u_t) dt 


tau = inf; % Time constant of the future discount in seconds. 
           % for t = tau seconds, the reward importance decays by
           % 100/e = 36.79 %

T = 5; % in seconds

dt = 1/100; % time step in seconds


sqdt = sqrt(dt);
s= ceil(T/dt);



[k, alpha, beta]= ctfhlqr(a,b,c,p,q,tau,dt,T);

sprintf('Limit Solution Found Using ctfhlqr program: %f %f', k{1}(1),k{1}(2))

% Compare to the stationary solution computed
% using the Matlab lqr program

[k1, s1, e1] = lqr(a,b,p,q);

sprintf('Limit Solution Found Using Matlab lqr program: %f %f', k1(1), k1(2))

% Here we run the system with the optimal controller
x= zeros(nx,s);
u = zeros(nx,s);
x(1,1)=1;
for t=1:s-1
  u(:,t)=- k{t}*x(:,t);
  dx = (a*x(:,t)  +u(:,t))*dt + c*randn(nx,1)*sqdt;
  x(:,t+1) = x(:,t)+ dx;
end




subplot(2,1,1)

plot([1:s]*dt,x(1,:))
hold on
plot([1:s]*dt,x(2,:),'r')
xlabel('Time (seconds)');
ylabel('X');

subplot(2,1,2)



plot([1:s]*dt,u)
xlabel('Time (seconds)');
ylabel('U');


